@@ -3,7 +3,7 @@ BEGIN {
   $Dancer::Plugin::Cache::CHI::AUTHORITY = 'cpan:YANICK';
 }
 {
-  $Dancer::Plugin::Cache::CHI::VERSION = '1.3.1';
+  $Dancer::Plugin::Cache::CHI::VERSION = '1.4.0';
 }
 # ABSTRACT: Dancer plugin to cache response content (and anything else)
 
@@ -12,15 +12,28 @@ use warnings;
 no warnings qw/ uninitialized /;
 
 use Dancer 1.1904 ':syntax';
-use Dancer::Plugin;
-use Dancer::Hook;
-use Dancer::Factory::Hook;
-use Dancer::Response;
-use Dancer::SharedData;
+
+my $dancer_version = int Dancer->VERSION;
 
 use Carp;
 use CHI;
 
+use Dancer::Plugin;
+
+use Moo;
+
+if ( $dancer_version < 2 ) {
+    require Dancer::Hook;
+    require Dancer::Factory::Hook;
+    require Dancer::Response;
+    require Dancer::SharedData;
+}
+else {
+    with 'Dancer::Plugin';
+
+    register_hook 'before_create_cache';
+}
+
 
 my %cache;     
 my $cache_page; # actually hold the ref to the args
@@ -45,7 +58,8 @@ hook after => sub {
 };
 
 register cache => sub {
-    return  $cache{$_[0]} ||= _create_cache( @_ );
+    shift if $dancer_version >= 2;
+    return  $cache{$_[0]//''} ||= _create_cache( @_ );
 };
 
 my $honor_no_cache = 0;
@@ -54,7 +68,12 @@ sub _create_cache {
     my $namespace = shift;
     my $args = shift || {};
 
-    Dancer::Factory::Hook->execute_hooks( 'before_create_cache' );
+    if ( $dancer_version < 2 ) {
+        Dancer::Factory::Hook->execute_hooks( 'before_create_cache' );
+    }
+    else {
+        execute_hook 'before_create_cache';
+    }
 
     my %setting = %{ plugin_setting() };
 
@@ -73,7 +92,11 @@ sub _create_cache {
 
 
 register check_page_cache => sub {
-    hook before => sub {
+    shift if $dancer_version >= 2;
+
+    my $hook = sub {
+        my $dsl = shift if $dancer_version >= 2;
+
         # Instead halt() now we use a more correct method - setting of a
         # response to Dancer::Response object for a more correct returning of
         # some HTTP headers (X-Powered-By, Server)
@@ -82,33 +105,59 @@ register check_page_cache => sub {
             or return;
 
         if ( $honor_no_cache ) {
-            my $req =  Dancer::SharedData->request;
+            $DB::single = 1;
+
+            my $req =  $dancer_version >=2  
+                    ? $dsl->request
+                    : Dancer::SharedData->request
+                    ;
 
             return if grep { 
                 # eval is there to protect from a regression in Dancer 1.31
                 # where headers can be undef
-                eval { $req->header($_) } eq 'no-cache' 
+                eval { $req->header($_) eq 'no-cache' }
             } qw/ Cache-Control Pragma /;
         }
 
-        Dancer::SharedData->response(
-            Dancer::Response->new(
-                ref $cached eq 'HASH'
-                ?
-                (
-                    status       => $cached->{status},
-                    headers      => $cached->{headers},
-                    content      => $cached->{content}
+        if ( $dancer_version < 2 ) {
+            Dancer::SharedData->response(
+                Dancer::Response->new(
+                    ref $cached eq 'HASH'
+                    ?
+                    (
+                        status       => $cached->{status},
+                        headers      => $cached->{headers},
+                        content      => $cached->{content}
+                    )
+                    :
+                    ( content => $cached )
                 )
-                :
-                ( content => $cached )
-            )
-        );
+            );
+        } else {
+            $dsl->response(
+                Dancer::Core::Response->new(
+                    is_halted => 1,
+                    ref $cached eq 'HASH'
+                    ?
+                    (
+                        status       => $cached->{status},
+                        headers      => $cached->{headers},
+                        content      => $cached->{content}
+                    )
+                    :
+                    ( content => $cached )
+                )
+            );
+        }
     };
+
+    hook before => $hook;
 };
 
 
 register cache_page => sub {
+    shift if $dancer_version >= 2;
+
     my ( $content, @args ) = @_;
 
     $cache_page = \@args;
@@ -119,27 +168,31 @@ register cache_page => sub {
 
 
 register cache_page_key => sub {
+    shift if $dancer_version >= 2;
     return $cache_page_key_generator->();
 };
 
 
 register cache_page_key_generator => sub {
+    shift if $dancer_version >= 2;
     $cache_page_key_generator = shift;
 };
 
 
 for my $method ( qw/ set get remove clear compute / ) {
     register 'cache_'.$method => sub {
+        shift if $dancer_version >= 2;
         return cache()->$method( @_ );
     }
 }
 
-Dancer::Factory::Hook->instance->install_hooks(qw/ before_create_cache /);
+Dancer::Factory::Hook->instance->install_hooks(qw/ before_create_cache /)
+    if $dancer_version < 2;
 
 
-register_plugin;
-
+register_plugin for_versions => [1,2];
 
+__END__
 
 =pod
 
@@ -149,7 +202,7 @@ Dancer::Plugin::Cache::CHI - Dancer plugin to cache response content (and anythi
 
 =head1 VERSION
 
-version 1.3.1
+version 1.4.0
 
 =head1 SYNOPSIS
 
@@ -321,7 +374,3 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
-
-__END__
-
